#pragma once
#pragma unmanaged

#ifdef RACELOGIC_VBOX_API_DLL_EXPORTS
#define EXPORTED_MEMBER __declspec(dllexport)
#else
#define EXPORTED_MEMBER __declspec(dllimport)
#endif

#include <stdio.h>
#include <vector>
#include <ctype.h>
#include "unmanagedSampleEventArgs.h"

namespace RacelogicSetupApiCpp 
{
	extern "C" 
	{
		class EXPORTED_MEMBER RacelogicSetupApiCpp
		{
		private:
			
		public:
			
			RacelogicSetupApiCpp() {};
			~RacelogicSetupApiCpp() {};

			/// <summary>   Enables parsing of the VBOX serial data. </summary>
			///
			/// <param name="onNewSample"></param> Event that is fired every time there is new data available
			/// <param name="channels">  [out] The list of all channel names. </param>
			/// <param name="serialnumbers">  [out] The list of all serialnumbers for associated channels. </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool BeginParse(void* (__stdcall *onNewSample) (UnmanagedSampleEventArgs args), std::vector<std::string> &channels, std::vector<int> &serialnumbers);

			/// <summary>   Connects to a VBOX. </summary>
			///
			/// <param name="portName">     Name of the port the unit is connected to. </param>
			/// <param name="serialNumber"> [out] The serial number of the connected unit. </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool Connect(std::string& portName, std::int32_t& serialNumber);

			/// <summary>   Disconnects from the VBOX and closes the port. </summary>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool Disconnect();

			/// <summary> Disables parsing of the VBOX serial data.
			void EndParse();

			/// <summary>   Gets a list of the serial numbers of any connected modules. </summary>
			///
			/// <param name="moduleSerialNumbers">  [out] The list of serial numbers of any connected modules. </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool GetConnectedModules(std::vector<std::int32_t>& moduleSerialNumbers);

			/// <summary>   Gets external battery voltage (power supply). </summary>
			///
			/// <param name="voltage">  [out] The voltage. </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool GetExternalBatteryVoltage(float& voltage);

			/// <summary>   Gets firmware version of the connected VBOX. </summary>
			///
			/// <param name="version">  [out] The version. </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool GetFirmwareVersion(std::string& version);

			/// <summary>   Gets internal battery voltage. </summary>
			///
			/// <param name="voltage">  [out] The voltage. </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool GetInternalBatteryVoltage(float& voltage);

			/// <summary>   Gets a list of all com port names on the current pc. </summary>
			///
			/// <returns>   An array of string. </returns>
			std::vector<std::string> GetPortNames();

			/// <summary>   Loads VBOX configuration file (*.RCF). </summary>
			///
			/// <param name="fileName"> Filename of the file. </param>
			/// <param name="error">    [out] The error (if fail). </param>
			///
			/// <returns>   true if it succeeds, false if it fails. </returns>
			bool LoadConfigurationFile(std::string fileName, std::string& error);
		};
	}
}
